/*
	to run this pianola: RunPianola();
	Small pianola example using the Acusthica sound library.
	Programmed to play 1 tone at a time (although Sphere can handle multiple sounds).
*/


RequireScript("lib/acusthica.js");
var M = new AcusthicaEngine(); //create a new acusthica object called 'M'
var sndArr=new Array(); //Array where we will hold all sounds
sndArr.push("flute01.ogg");
sndArr.push("latin_brass01.ogg");
sndArr.push("violin_pizzicato01.ogg");

//Preload sounds in the array sndArr[]
function PreLoadSounds(){
	for(var i=0;i<sndArr.length;i++){
		M.sfxLoad(sndArr[i]);
	}
}

/**
 * Play a note, but if we are already playing that note, dont play it again, or it will sound like arpeggio.
 */
function playnote(note,oldnote,snd){
        // if(M.Sounds[snd].isPlaying() && M.Sounds[snd].getPitch() == M.octave2pitch(note)) //Doesnt work, float rounding bugs in JS
	if( note==oldnote && M.Sounds[sndArr[snd]].isPlaying() )
                return; //SUSTAIN
        M.sfx(sndArr[snd],0,0,note);
}

/**
 * Main function
 */
function RunPianola(){
	PreLoadSounds();
	var snd = 0; //current sound
	var note = ""; //current note that is playing
	var oldnote = ""; //previous note played
	var nokeys=true;
	var active=true;
	while(active){
        	nokeys=true;
		//This loop is a little complicated because I wanted to be able to sustain the note while you keep pressing it,
		//without key-retrigger getting in the way (which would keep the AreKeysLeft() true for ever)
        	while (AreKeysLeft()){
                	nokeys=false;
			newnote="";
                	switch (GetKey()){
                        	case KEY_Q: newnote='C'; break;
                        	case KEY_W: newnote='D'; break;
				case KEY_E: newnote='E'; break;
	                        case KEY_R: newnote='F'; break;
        	                case KEY_T: newnote='G'; break;
                	        case KEY_Y: newnote='A'; break;
                        	case KEY_U: newnote='B'; break;
	                        case KEY_I: newnote='C2'; break;

        	                case KEY_2: newnote='C#'; break;
                	        case KEY_3: newnote='D#'; break;
                        	case KEY_5: newnote='F#'; break;
	                        case KEY_6: newnote='G#'; break;
        	                case KEY_7: newnote='A#'; break;

                	        case KEY_Z: newnote='c'; break;
                        	case KEY_X: newnote='d'; break;
	                        case KEY_C: newnote='e'; break;
        	                case KEY_V: newnote='f'; break;
                	        case KEY_B: newnote='g'; break;
	                        case KEY_N: newnote='a'; break;
                        	case KEY_M: newnote='b'; break;

	                        case KEY_S: newnote='c#'; break;
        	                case KEY_D: newnote='d#'; break;
                	        case KEY_G: newnote='f#'; break;
                        	case KEY_H: newnote='g#'; break;
	                        case KEY_J: newnote='a#'; break;

				//swap between sounds
				case KEY_PAGEUP: if(++snd>sndArr.length-1) snd=sndArr.length-1; break;
				case KEY_PAGEDOWN: if(--snd<0) snd=0; break;

	                        case KEY_ESCAPE: active=false;
        	                default: newnote='X'; break;
                	}

			//A note is pressed, lets process it.
			if (newnote) {
				if (newnote == 'X') {
					//Weird key has been pressed, just stop playing sounds.
					if(M.Sounds[sndArr[snd]].isPlaying()) {M.Sounds[sndArr[snd]].stop(); note="";} //RELEASE
				}else{
					oldnote=note;
					note=newnote;
					playnote(note,oldnote,snd); //ATTACK
				}
				//break;
			}

        	}

		//No note keys are being pressed, RELEASE. todo: DECAY: maybe fade sound
        	if(nokeys && M.Sounds[sndArr[snd]].isPlaying()) { M.Sounds[sndArr[snd]].stop(); oldnote=note="";}

        	GetSystemFont().drawText(20,15, "PLAYING="+note +" oldnote="+oldnote+" snd="+snd+" "+sndArr[snd]+" "+GetTime());
        	GetSystemFont().drawText(95,1, "Pianola v1.0 using Acusthica.");
        	GetSystemFont().drawText(5,GetScreenHeight()-12, "Press keys on keyboard to play. ESC to exit.");
        	FlipScreen();

        	time=GetTime()+100; //10 fps
        	while(GetTime()<time){};
	}
}


